﻿Imports System.IO

Public Class MainForm

    Private strInvalidCharactersArray() As String = {"<", ">", ":", "/", "\", "|", "?", "*", Chr(34)}
    Private intStandardFoldersArray() As String = {"Base.rte", "Browncoats.rte", "Coalition.rte", "Dummy.rte", "Imperatus.rte",
                                           "Metagames.rte", "Missions.rte", "Ronin.rte", "Techion.rte", "Tutorial.rte", "Presets"}
    Private Const intArrayLength As Integer = 149
    Private strModPathArray(intArrayLength, 1) As String
    Private Const intModName As Integer = 0
    Private Const intModPath As Integer = 1

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Load settings
        If Not My.Settings.strCCFolder = "" Then txtFolderPath.Text = My.Settings.strCCFolder
        If My.Settings.bolDeletePermanently = True Then chkDeletePermanently.Checked = True

        ' Refresh lists
        refreshLists()
    End Sub

    Private Sub btnSelectFolder_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSelectFolder.Click

        ' Declare the folder browser.
        Dim FolderBrowser As New FolderBrowserDialog

        ' Give instructions.
        FolderBrowser.Description = "Please select the main Cortex Command folder"

        ' Hide the "New folder" button.
        FolderBrowser.ShowNewFolderButton = False

        ' Open the window, check for result.
        If FolderBrowser.ShowDialog = Windows.Forms.DialogResult.OK Then

            'Put the selected path into the text box.
            Me.txtFolderPath.Text = FolderBrowser.SelectedPath

            'Save the folder location for next time.
            My.Settings.strCCFolder = FolderBrowser.SelectedPath

        End If

        ' Refresh the lists
        refreshLists()

    End Sub

    Private Sub btnRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRefresh.Click

        ' Refresh lists
        refreshLists()
    End Sub

    Private Sub listAllFolders()

        'Clear the lists to avoid duplicates.
        lstActiveMods.Items.Clear()
        lstMods.Items.Clear()

        ' Declare a new Directoryinfo, and set it to the main CC folder.
        Dim di As New DirectoryInfo(txtFolderPath.Text)

        ' For every subdirectory in the folder, add its name to the listbox.
        Dim count As Integer = 0
        For Each subdi As DirectoryInfo In di.GetDirectories

            ' Check if the folder is default CC content.
            Dim skip As Boolean = False
            Dim i As String
            For Each i In intStandardFoldersArray
                If intStandardFoldersArray.Contains(subdi.Name) Then skip = True
            Next


            ' For every subdirectory in the folder, add its name to the listbox.
            If skip = False Then

                ' Remove .rte from the name if neccessary and put the mod into the correct listbox.
                Dim modName As String

                If subdi.Name.EndsWith(".rte") Then
                    modName = subdi.Name.Substring(0, subdi.Name.Length - 4)
                    lstActiveMods.Items.Add(modName)
                Else
                    modName = subdi.Name
                    lstMods.Items.Add(modName)
                End If

                ' Put the name of the mod and its path in an array.
                strModPathArray(count, intModName) = subdi.Name
                strModPathArray(count, intModPath) = txtFolderPath.Text + "\" + subdi.Name
                count = count + 1
            End If
        Next

    End Sub

    Private Sub lstMods_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles lstMods.DoubleClick

        'Enable the mod
        enableMod()

    End Sub

    Private Sub enableMod()

        ' Cancel if there are no items in the list, or if no item is selected
        If lstMods.Items.Count() < 1 Or lstMods.SelectedIndex = -1 Then Exit Sub

        Dim arrayNumber As String = ""

        ' Find the path to the selected mod
        For count = 0 To strModPathArray.Length - 1
            If strModPathArray(count, intModName) = lstMods.SelectedItem Then
                arrayNumber = count
                Exit For
            End If
        Next

        ' Add .rte to the end of the folder name for the mod
        Dim path = strModPathArray(arrayNumber, intModPath)
        Dim name = strModPathArray(arrayNumber, intModName)

        My.Computer.FileSystem.RenameDirectory(path, name + ".rte")

        ' Copy the selected item to the other box when doubleclicked
        lstActiveMods.Items.Add(lstMods.Items.Item(lstMods.SelectedIndex))

        ' Remove the item from the current list
        lstMods.Items.RemoveAt(lstMods.SelectedIndex)

        ' Refresh lists
        refreshLists()
    End Sub

    Private Sub lstActiveMods_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles lstActiveMods.DoubleClick

        ' Disable mod
        disableMod()
    End Sub

    Private Sub disableMod()

        ' Cancel if there are no items in the list, or if no item is selected
        If lstActiveMods.Items.Count() < 1 Or lstActiveMods.SelectedIndex = -1 Then Exit Sub

        ' Find the path to the selected mod
        Dim arrayNumber As String = ""

        For count = 0 To intArrayLength
            If Not strModPathArray(count, intModName).Length < 5 Then
                If strModPathArray(count, intModName).Substring(0, strModPathArray(count, intModName).Length - 4) = lstActiveMods.SelectedItem.ToString Then
                    arrayNumber = count
                    Exit For
                End If
            End If
        Next

        ' Add .rte to the end of the folder name for the mod
        Dim path As String = strModPathArray(arrayNumber, intModPath)
        Dim name As String = strModPathArray(arrayNumber, intModName)

        Dim test As String = name.Substring(0, name.Length - 4)

        My.Computer.FileSystem.RenameDirectory(path, name.Substring(0, name.Length - 4))

        ' Copy the selected item to the other box when doubleclicked
        lstMods.Items.Add(lstActiveMods.Items.Item(lstActiveMods.SelectedIndex))

        ' Remove the item from the current list
        lstActiveMods.Items.RemoveAt(lstActiveMods.SelectedIndex)

        ' Refresh lists
        refreshLists()

    End Sub

    Private Sub refreshLists()

        ' Just cancel if the path has not been specified
        If txtFolderPath.Text = "" Then Exit Sub

        ' Declare the path to the Cortex Command .exe file
        Dim pathToExe As String = txtFolderPath.Text + "/Cortex Command.exe"

        ' Show an error message if Cortex Command.exe does not exist within the folder, and clear some boxes and the settings
        If Dir(pathToExe) = "" Then
            MsgBox("Please specify the Cortex Command folder.", MsgBoxStyle.OkOnly, "Error")
            txtFolderPath.Text = ""
            lstMods.Items.Clear()
            lstActiveMods.Items.Clear()
            My.Settings.strCCFolder = ""
            Exit Sub
        End If



        ' Declare the path to the presets folder
        Dim strPresetsPath As String = txtFolderPath.Text + "\Presets"

        ' Cancel the rest of the sub if it is not needed to avoid exceptions
        If Not My.Computer.FileSystem.DirectoryExists(strPresetsPath) Then
            listAllFolders()
            Exit Sub
        End If

        'Clear the list to avoid duplicates
        lstPresets.Items.Clear()

        ' List all presets and put them in the presets listbox
        Dim finfo As New IO.DirectoryInfo(strPresetsPath)
        For Each fi In finfo.GetFiles

            If fi.Name.EndsWith(".preset") Then
                lstPresets.Items.Add(fi.Name.Substring(0, fi.Name.Length - 7))
            End If
        Next

        ' List folders in their appropriate listboxes
        listAllFolders()
    End Sub

    Private Sub btnDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDelete.Click

        ' Cancel if there are no items in the list, or if no item is selected.
        If lstMods.Items.Count() < 1 Or lstMods.SelectedIndex = -1 Then Exit Sub

        Dim arrayNumber As Integer = 0

        ' Find the path to the selected mod.
        For i = 0 To strModPathArray.Length - 1
            If strModPathArray(i, intModName) = lstMods.SelectedItem Then
                arrayNumber = i
                Exit For
            End If
        Next

        'Check if the folder exists to avoid exceptions
        If Not My.Computer.FileSystem.DirectoryExists(strModPathArray(arrayNumber, intModPath)) Then
            MsgBox("Folder could not be found, is the list updated?", MsgBoxStyle.Information, "Error")
        End If

        ' Check if it should be deleted permanently or just recycled, and then do it (the system asks for confirmation, so we don't)
        ' We might need a progress bar for mods as they can get huge, and we make sure there is no exception if the user decides to cancel
        ' We make sure eventual exceptions are handled
        Try
            If chkDeletePermanently.Checked = False Then
                My.Computer.FileSystem.DeleteDirectory(strModPathArray(arrayNumber, intModPath), FileIO.UIOption.AllDialogs, _
                                                       FileIO.RecycleOption.SendToRecycleBin, FileIO.UICancelOption.DoNothing)
            Else
                My.Computer.FileSystem.DeleteDirectory(strModPathArray(arrayNumber, intModPath), FileIO.UIOption.AllDialogs, _
                                                       FileIO.RecycleOption.DeletePermanently, FileIO.UICancelOption.DoNothing)
            End If

        Catch ex As Exception
            MsgBox("An exception occured while trying to save the preset, please post the following error message in the " + _
                   "Data Realms Fan Forums mod manager thread along with what you were trying to do:" + vbCrLf + ex.Message)
        Finally

            ' Update the list
            refreshLists()

        End Try
    End Sub

    Private Sub lstMods_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles lstMods.GotFocus
        btnDelete.Enabled = True
        lstActiveMods.ClearSelected()
    End Sub

    Private Sub lstActiveMods_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles lstActiveMods.GotFocus
        btnDelete.Enabled = False
        lstMods.ClearSelected()
    End Sub

    Private Sub chkDeletePermanently_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkDeletePermanently.CheckedChanged
        My.Settings.bolDeletePermanently = chkDeletePermanently.Checked
    End Sub

    Private Sub btnSavePreset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSavePreset.Click

        ' Cancel if there is no path specified
        If txtFolderPath.Text = "" Then Exit Sub

        ' Put it all in a "Try" to catch exceptions, as this code has a high chance of producing one
        Try

            ' Cancel if there are no items in the list
            If lstActiveMods.Items.Count() < 1 Then
                MsgBox("No mods selected!", MsgBoxStyle.OkOnly, "Error")
                Exit Sub
            End If

            ' Ask for a name for the preset
            Dim strFileName As String = InputBox("Enter the name of the preset: ", "Name")

            ' Save the path to it for later
            Dim strFilePath As String = txtFolderPath.Text + "\Presets\" + strFileName + ".preset"

            ' Create the presets folder if it does not exist.
            If Not My.Computer.FileSystem.DirectoryExists(txtFolderPath.Text + "\Presets") Then
                My.Computer.FileSystem.CreateDirectory(txtFolderPath.Text + "\Presets")
            End If

            ' Make sure the filename is valid to avoid exceptions
            For i = 0 To 8
                If strFileName.Contains(strInvalidCharactersArray(i)) Or strFileName = "" Or + _
                    My.Computer.FileSystem.FileExists(strFilePath) Then

                    ' The name is invalid, so we tell the user and cancel the operation
                    MsgBox("The name is either invalid, or a file by that name already exists.", MsgBoxStyle.OkOnly, "Error")
                    Exit Sub
                End If
            Next

            ' Write all the mods to a text file
            For i = 0 To lstActiveMods.Items.Count - 1
                My.Computer.FileSystem.WriteAllText(strFilePath, lstActiveMods.Items.Item(i) + ",", True)
            Next

            ' If there is an error while saving the preset, it should abort and tell the user instead of crashing
        Catch ex As Exception
            MsgBox("An exception occured while trying to save the preset, please post the following error message in the " + _
                   "Data Realms Fan Forums mod manager thread along with what you were trying to do:" + vbCrLf + ex.Message)

        End Try

        ' Refresh lists again
        refreshLists()

    End Sub

    Private Sub btnDeletePreset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDeletePreset.Click

        ' Cancel if there are no items in the list, or if no item is selected.
        If lstPresets.Items.Count() < 1 Or lstPresets.Items.Count = -1 Then Exit Sub

        ' Declare the file path
        Dim strPresetPath As String = txtFolderPath.Text + "\Presets\" + _lstPresets.SelectedItem.ToString + ".preset"

        'Check if the file exists to avoid exceptions
        If Not My.Computer.FileSystem.FileExists(txtFolderPath.Text + "\Presets\" + _lstPresets.SelectedItem.ToString + ".preset") Then
            MsgBox("File could not be found, is the list updated?", MsgBoxStyle.Information, "Error")
        End If

        ' Check if it should be deleted permanently or just recycled, and then do it (the system asks for confirmation, so we don't)
        ' We make sure there is no exception if the user decides to cancel
        If chkDeletePermanently.Checked = False Then
            My.Computer.FileSystem.DeleteFile(strPresetPath, FileIO.UIOption.AllDialogs, _
                                              FileIO.RecycleOption.SendToRecycleBin, FileIO.UICancelOption.DoNothing)
        Else
            My.Computer.FileSystem.DeleteFile(strPresetPath, FileIO.UIOption.AllDialogs, _
                                              FileIO.RecycleOption.DeletePermanently, FileIO.UICancelOption.DoNothing)
        End If

        ' Update the list
        refreshLists()
    End Sub

    Private Sub btnLoadPreset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnLoadPreset.Click

        ' Cancel if there is no path specified
        If txtFolderPath.Text = "" Then Exit Sub

        ' Cancel if there are no items in the list, or if no item is selected
        If lstPresets.Items.Count < 1 Or lstPresets.SelectedIndex = -1 Then Exit Sub

        ' Remember what preset is selected
        Dim selectedpreset As Integer = lstPresets.SelectedIndex

        ' Make sure the lists are updated
        refreshLists()

        ' Set the selected preset again (because refreshlists clears it)
        lstPresets.SelectedIndex = selectedpreset

        ' Declare the file path
        Dim strPresetsPath As String = txtFolderPath.Text + "\Presets\" + lstPresets.SelectedItem.ToString + ".preset"

        'Check if the file exists to avoid exceptions
        If Not My.Computer.FileSystem.FileExists(txtFolderPath.Text + "\Presets\" + _lstPresets.SelectedItem.ToString + ".preset") Then
            MsgBox("File could not be found, is the list updated?", MsgBoxStyle.Information, "Error")
        End If

        ' List all presets and put them in the presets listbox
        Dim finfo2 As New IO.DirectoryInfo(txtFolderPath.Text + "\Presets")
        For Each fi In finfo2.GetFiles

            If fi.Name.EndsWith(".preset") Then
                lstPresets.Items.Add(fi.Name.Substring(0, fi.Name.Length - 7))
            End If
        Next

        ' Make sure the file isn't empty, starting by declaring a boolean
        Dim bolEmpty As Boolean = True

        ' Declare an array of undefined size to store mod names in
        Dim strModNameArray() As String

        ' Use TextFieldParser to get each mod
        Using TextReader As New Microsoft.VisualBasic.FileIO.TextFieldParser(strPresetsPath)
            TextReader.TextFieldType = FileIO.FieldType.Delimited
            TextReader.SetDelimiters(",")

            ' Save the name of the mod to an array
            Dim i As Integer = 0

            While Not TextReader.EndOfData
                Dim modNames As String() = TextReader.ReadFields

                For Each modName In modNames

                    If Not modName = "" Then

                        ' Here we increase the size of our array and store the mod name.
                        ' We also make sure to mark the file as "not empty"
                        ReDim Preserve strModNameArray(i)
                        strModNameArray(i) = modName
                        bolEmpty = False
                        i = i + 1
                    End If
                Next
            End While
        End Using

        ' Cancel if the file is empty
        If bolEmpty = True Then
            MsgBox("No mods found in the preset", MsgBoxStyle.OkOnly, "Error")
            Exit Sub
        End If

        ' Now that we have an array, we will disable all mods, and reenable those in the list

        ' Disable all mods
        While lstActiveMods.Items.Count > 0
            lstActiveMods.SelectedIndex = 0
            disableMod()
        End While

        ' Here we check if the mod is in the disabled mods list, and if so, activate it
        ' Using the array without VB being sure it has values creates som warnings, but whatever
        For i = 0 To strModNameArray.GetUpperBound(0)
            If lstMods.Items.Contains(strModNameArray(i)) Then
                lstMods.SelectedIndex = lstMods.Items.IndexOf(strModNameArray(i))
                enableMod()
            End If
        Next

        ' Refresh lists
        refreshLists()

    End Sub

    Private Sub btnOpenFolder_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOpenFolder.Click

        ' Make sure a path is specified, and that it exist
        If txtFolderPath.Text = "" Or My.Computer.FileSystem.DirectoryExists(txtFolderPath.Text) = False Then Exit Sub

        ' Open it in explorer
        Process.Start("explorer.exe", txtFolderPath.Text)
    End Sub

End Class
